"""
The API allows convert textual content to audio format in the following formats:
"""
mono_8khz_8bit = "8khz_8bit_mono"
"""8 kHz, 8 Bit, Mono"""

stereo_8khz_8bit = "8khz_8bit_stereo"
"""8 kHz, 8 Bit, Stereo"""

mono_8khz_16bit = "8khz_16bit_mono"
"""8 kHz, 16 Bit, Mono"""

stereo_8khz_16bit = "8khz_16bit_stereo"
"""8 kHz, 16 Bit, Stereo"""

mono_11khz_8bit = "11khz_8bit_mono"
"""11 kHz, 8 Bit, Mono"""

stereo_11khz_8bit = "11khz_8bit_stereo"
"""11 kHz, 8 Bit, Stereo"""

mono_11khz_16bit = "11khz_16bit_mono"
"""11 kHz, 16 Bit, Mono"""

stereo_11khz_16bit = "11khz_16bit_stereo"
"""11 kHz, 16 Bit, Stereo"""

mono_12khz_8bit = "12khz_8bit_mono"
"""12 kHz, 8 Bit, Mono"""

stereo_12khz_8bit = "12khz_8bit_stereo"
"""12 kHz, 8 Bit, Stereo"""

mono_12khz_16bit = "12khz_16bit_mono"
"""12 kHz, 16 Bit, Mono"""

stereo_12khz_16bit = "12khz_16bit_stereo"
"""12 kHz, 16 Bit, Stereo"""

mono_16khz_8bit = "16khz_8bit_mono"
"""16 kHz, 8 Bit, Mono"""

stereo_16khz_8bit = "16khz_8bit_stereo"
"""16 kHz, 8 Bit, Stereo"""

mono_16khz_16bit = "16khz_16bit_mono"
"""16 kHz, 16 Bit, Mono"""

stereo_16khz_16bit = "16khz_16bit_stereo"
"""16 kHz, 16 Bit, Stereo"""

mono_22khz_8bit = "22khz_8bit_mono"
"""22 kHz, 8 Bit, Mono"""

stereo_22khz_8bit = "22khz_8bit_stereo"
"""22 kHz, 8 Bit, Stereo"""

mono_22khz_16bit = "22khz_16bit_mono"
"""22 kHz, 16 Bit, Mono"""

stereo_22khz_16bit = "22khz_16bit_stereo"
"""22 kHz, 16 Bit, Stereo"""

mono_24khz_8bit = "24khz_8bit_mono"
"""24 kHz, 8 Bit, Mono"""

stereo_24khz_8bit = "24khz_8bit_stereo"
"""24 kHz, 8 Bit, Stereo"""

mono_24khz_16bit = "24khz_16bit_mono"
"""24 kHz, 16 Bit, Mono"""

stereo_24khz_16bit = "24khz_16bit_stereo"
"""24 kHz, 16 Bit, Stereo"""

mono_32khz_8bit = "32khz_8bit_mono"
"""32 kHz, 8 Bit, Mono"""

stereo_32khz_8bit = "32khz_8bit_stereo"
"""32 kHz, 8 Bit, Stereo"""

mono_32khz_16bit = "32khz_16bit_mono"
"""32 kHz, 16 Bit, Mono"""

stereo_32khz_16bit = "32khz_16bit_stereo"
"""32 kHz, 16 Bit, Stereo"""

mono_44khz_8bit = "44khz_8bit_mono"
"""44 kHz, 8 Bit, Mono"""

stereo_44khz_8bit = "44khz_8bit_stereo"
"""44 kHz, 8 Bit, Stereo"""

mono_44khz_16bit = "44khz_16bit_mono"
"""44 kHz, 16 Bit, Mono"""

stereo_44khz_16bit = "44khz_16bit_stereo"
"""44 kHz, 16 Bit, Stereo"""

mono_48khz_8bit = "48khz_8bit_mono"
"""48 kHz, 8 Bit, Mono"""

stereo_48khz_8bit = "48khz_8bit_stereo"
"""48 kHz, 8 Bit, Stereo"""

mono_48khz_16bit = "48khz_16bit_mono"
"""48 kHz, 16 Bit, Mono"""

stereo_48khz_16bit = "48khz_16bit_stereo"
"""48 kHz, 16 Bit, Stereo"""

mono_alaw_8khz = "alaw_8khz_mono"
"""ALaw, 8 kHz, Mono"""

stereo_alaw_8khz = "alaw_8khz_stereo"
"""ALaw, 8 kHz, Stereo"""

mono_alaw_11khz = "alaw_11khz_mono"
"""ALaw, 11 kHz, Mono"""

stereo_alaw_11khz = "alaw_11khz_stereo"
"""ALaw, 11 kHz, Stereo"""

mono_alaw_22khz = "alaw_22khz_mono"
"""ALaw, 22 kHz, Mono"""

stereo_alaw_22khz = "alaw_22khz_stereo"
"""ALaw, 22 kHz, Stereo"""

mono_alaw_44khz = "alaw_44khz_mono"
"""ALaw, 44 kHz, Mono"""

stereo_alaw_44khz = "alaw_44khz_stereo"
"""ALaw, 44 kHz, Stereo"""

mono_ulaw_8khz = "ulaw_8khz_mono"
"""uLaw, 8 kHz, Mono"""

stereo_ulaw_8khz = "ulaw_8khz_stereo"
"""uLaw, 8 kHz, Stereo"""

mono_ulaw_11khz = "ulaw_11khz_mono"
"""uLaw, 11 kHz, Mono"""

stereo_ulaw_11khz = "ulaw_11khz_stereo"
"""uLaw, 11 kHz, Stereo"""

mono_ulaw_22khz = "ulaw_22khz_mono"
"""uLaw, 22 kHz, Mono"""

stereo_ulaw_22khz = "ulaw_22khz_stereo"
"""uLaw, 22 kHz, Stereo"""

mono_ulaw_44khz = "ulaw_44khz_mono"
"""uLaw, 44 kHz, Mono"""

stereo_ulaw_44khz = "ulaw_44khz_stereo"
"""uLaw, 44 kHz, Stereo"""



class ar_eg:
    """Arabic (Egypt)"""

    Oda = "Oda"
    """Female. Default voice for this language."""


class ar_sa:
    """Arabic (Saudi Arabia)"""

    Salim = "Salim"
    """Male. Default voice for this language."""


class bg_bg:
    """Bulgarian"""

    Dimo = "Dimo"
    """Male. Default voice for this language."""


class ca_es:
    """Catalan"""

    Rut = "Rut"
    """Female. Default voice for this language."""


class zh_cn:
    """Chinese (China)"""

    Luli = "Luli"
    """Female. Default voice for this language."""

    Shu = "Shu"
    """Female."""

    Chow = "Chow"
    """Female."""

    Wang = "Wang"
    """Male."""


class zh_hk:
    """Chinese (Hong Kong)"""

    Jia = "Jia"
    """Female. Default voice for this language."""

    Xia = "Xia"
    """Female."""

    Chen = "Chen"
    """Male."""


class zh_tw:
    """Chinese (Taiwan)"""

    Akemi = "Akemi"
    """Female. Default voice for this language."""

    Lin = "Lin"
    """Female."""

    Lee = "Lee"
    """Male."""


class hr_hr:
    """Croatian"""

    Nikola = "Nikola"
    """Male. Default voice for this language."""


class cs_cz:
    """Czech"""

    Josef = "Josef"
    """Male. Default voice for this language."""


class da_dk:
    """Danish"""

    Freja = "Freja"
    """Female. Default voice for this language."""


class nl_be:
    """Dutch (Belgium)"""

    Daan = "Daan"
    """Male. Default voice for this language."""


class nl_nl:
    """Dutch (Netherlands)"""

    Lotte = "Lotte"
    """Female. Default voice for this language."""

    Bram = "Bram"
    """Male."""


class en_au:
    """English (Australia)"""

    Zoe = "Zoe"
    """Female. Default voice for this language."""

    Isla = "Isla"
    """Female."""

    Evie = "Evie"
    """Female."""

    Jack = "Jack"
    """Male."""


class en_ca:
    """English (Canada)"""

    Rose = "Rose"
    """Female. Default voice for this language."""

    Clara = "Clara"
    """Female."""

    Emma = "Emma"
    """Female."""

    Mason = "Mason"
    """Male."""


class en_gb:
    """English (Great Britain)"""

    Alice = "Alice"
    """Female. Default voice for this language."""

    Nancy = "Nancy"
    """Female."""

    Lily = "Lily"
    """Female."""

    Harry = "Harry"
    """Male."""


class en_in:
    """English (India)"""

    Eka = "Eka"
    """Female. Default voice for this language."""

    Jai = "Jai"
    """Female."""

    Ajit = "Ajit"
    """Male."""


class en_ie:
    """English (Ireland)"""

    Oran = "Oran"
    """Male. Default voice for this language."""


class en_us:
    """English (United States)"""

    Linda = "Linda"
    """Female. Default voice for this language."""

    Amy = "Amy"
    """Female."""

    Mary = "Mary"
    """Female."""

    John = "John"
    """Male."""

    Mike = "Mike"
    """Male."""


class fi_fi:
    """Finnish"""

    Aada = "Aada"
    """Female. Default voice for this language."""


class fr_ca:
    """French (Canada)"""

    Emile = "Emile"
    """Female. Default voice for this language."""

    Olivia = "Olivia"
    """Female."""

    Logan = "Logan"
    """Female."""

    Felix = "Felix"
    """Male."""


class fr_fr:
    """French (France)"""

    Bette = "Bette"
    """Female. Default voice for this language."""

    Iva = "Iva"
    """Female."""

    Zola = "Zola"
    """Female."""

    Axel = "Axel"
    """Male."""


class fr_ch:
    """French (Switzerland)"""

    Theo = "Theo"
    """Male. Default voice for this language."""


class de_at:
    """German (Austria)"""

    Lukas = "Lukas"
    """Male. Default voice for this language."""


class de_de:
    """German (Germany)"""

    Hanna = "Hanna"
    """Female. Default voice for this language."""

    Lina = "Lina"
    """Female."""

    Jonas = "Jonas"
    """Male."""


class de_ch:
    """German (Switzerland)"""

    Tim = "Tim"
    """Male. Default voice for this language."""


class el_gr:
    """Greek"""

    Neo = "Neo"
    """Male. Default voice for this language."""


class he_il:
    """Hebrew"""

    Rami = "Rami"
    """Male. Default voice for this language."""


class hi_in:
    """Hindi"""

    Puja = "Puja"
    """Female. Default voice for this language."""

    Kabir = "Kabir"
    """Male."""


class hu_hu:
    """Hungarian"""

    Mate = "Mate"
    """Male. Default voice for this language."""


class id_id:
    """Indonesian"""

    Intan = "Intan"
    """Male. Default voice for this language."""


class it_it:
    """Italian"""

    Bria = "Bria"
    """Female. Default voice for this language."""

    Mia = "Mia"
    """Female."""

    Pietro = "Pietro"
    """Male."""


class ja_jp:
    """Japanese"""

    Hina = "Hina"
    """Female. Default voice for this language."""

    Airi = "Airi"
    """Female."""

    Fumi = "Fumi"
    """Female."""

    Akira = "Akira"
    """Male."""


class ko_kr:
    """Korean"""

    Nari = "Nari"
    """Female. Default voice for this language."""


class ms_my:
    """Malay"""

    Aqil = "Aqil"
    """Male. Default voice for this language."""


class nb_no:
    """Norwegian"""

    Marte = "Marte"
    """Female. Default voice for this language."""

    Erik = "Erik"
    """Male."""


class pl_pl:
    """Polish"""

    Julia = "Julia"
    """Female. Default voice for this language."""

    Jan = "Jan"
    """Male."""


class pt_br:
    """Portuguese (Brazil)"""

    Marcia = "Marcia"
    """Female. Default voice for this language."""

    Ligia = "Ligia"
    """Female."""

    Yara = "Yara"
    """Female."""

    Dinis = "Dinis"
    """Male."""


class pt_pt:
    """Portuguese (Portugal)"""

    Leonor = "Leonor"
    """Female. Default voice for this language."""


class ro_ro:
    """Romanian"""

    Doru = "Doru"
    """Male. Default voice for this language."""


class ru_ru:
    """Russian"""

    Olga = "Olga"
    """Female. Default voice for this language."""

    Marina = "Marina"
    """Female."""

    Peter = "Peter"
    """Male."""


class sk_sk:
    """Slovak"""

    Beda = "Beda"
    """Male. Default voice for this language."""


class sl_si:
    """Slovenian"""

    Vid = "Vid"
    """Male. Default voice for this language."""


class es_mx:
    """Spanish (Mexico)"""

    Juana = "Juana"
    """Female. Default voice for this language."""

    Silvia = "Silvia"
    """Female."""

    Teresa = "Teresa"
    """Female."""

    Jose = "Jose"
    """Male."""


class es_es:
    """Spanish (Spain)"""

    Camila = "Camila"
    """Female. Default voice for this language."""

    Sofia = "Sofia"
    """Female."""

    Luna = "Luna"
    """Female."""

    Diego = "Diego"
    """Male."""


class sv_se:
    """Swedish"""

    Molly = "Molly"
    """Female. Default voice for this language."""

    Hugo = "Hugo"
    """Male."""


class ta_in:
    """Tamil"""

    Sai = "Sai"
    """Male. Default voice for this language."""


class th_th:
    """Thai"""

    Ukrit = "Ukrit"
    """Male. Default voice for this language."""


class tr_tr:
    """Turkish"""

    Omer = "Omer"
    """Male. Default voice for this language."""


class vi_vn:
    """Vietnamese"""

    Chi = "Chi"
    """Male. Default voice for this language."""



"""
The Voice RSS Text-to-Speech (TTS) API allows conversion of textual content to speech easier than ever. 

Website: https://www.voicerss.org/
"""


from pathlib import Path
from typing import Any, Optional

import requests

from .. import logger
from . import formats
from . import languages
from . import codecs
from . import voices


class VoiceRSS:
    """
    The Voice RSS Text-to-Speech (TTS) API
    """

    URL = "http://api.voicerss.org/"
    """the url of this api"""

    hl = languages
    """summary of all languages this api supports"""

    v = voices
    """summary of all voices of the selected language this api supports"""

    c = codecs
    """summary of all codecs this api supports"""

    f = formats
    """summary of all formats this api supports"""

    def __init__(
        self,
        api_key: str,
        hl: str,
        v: Optional[str] = None,
        r: Optional[int] = None,
        c: Optional[str] = None,
        f: Optional[str] = None,
        ssml: Optional[bool] = None,
        b64: Optional[bool] = None,
    ) -> None:
        """
        You can set default values to use for all

        :param api_key: Required. The API key. Get your own API key here https://www.voicerss.org/login.aspx
        :param hl: Required. The textual content language.
            Allows values: see Languages.
        :param v: Required. The speech voice.
            Allows values: see Voices.
            Default value: depends on a language.
        :param r: Optional. The speech rate (speed).
            Allows values: from -10 (slowest speed) up to 10 (fastest speed).
            Default value: 0 (normal speed).
        :param c: Optional. The speech audio codec.
            Allows values: see Audio Codecs.
            Default value: WAV.
        :param f: Optional. The speech audio formats.
            Allows values: see Audio Formats.
            Default value: 8khz_8bit_mono.
        :param ssml: Optional. The SSML textual content format (see SSML documentation).
            Allows values: true and false.
            Default value: false.
        :param b64: Optional. Defines output as a Base64 string format (for an internet browser playing).
            Allows values: true and false.
            Default value: false.
        """
        self.api_key = api_key
        self.lang = hl
        self.voice = v
        self.rate = r
        self.codec = c
        self.format = f
        self.ssml = ssml
        self.b64 = b64

    def _request(self, **params: Any) -> bytes | None:
        params["key"] = self.api_key
        r = requests.request("POST", self.URL, params=params)
        data = r.content
        if data.startswith(b"ERROR: "):
            logger.error(
                f"A request to the {self.__class__.__name__} API was unsuccessful: {data.decode()}"
            )
            return

        return data

    def to_file(
        self,
        src: str,
        out: Path | str,
        hl: Optional[str] = None,
        v: Optional[str] = None,
        r: Optional[int] = None,
        c: Optional[str] = None,
        f: Optional[str] = None,
        *,
        ssml: Optional[bool] = None,
        b64: Optional[bool] = None,
    ) -> Path | None:
        """
        :param src: The textual content for converting to speech (length limited by 100KB).
        :param out: The path to save the result to.
            If the file exists, it will be overwritten.
            Note: The file must match the codec.
        :param hl: Mandatory. The textual content language.
            Allows values: see Languages.
        :param v: Optional. The speech voice.
            Allows values: see Voices.
            Default value: depends on a language.
        :param r: Optional. The speech rate (speed).
            Allows values: from -10 (slowest speed) up to 10 (fastest speed).
            Default value: 0 (normal speed).
        :param c: Optional. The speech audio codec.
            Allows values: see Audio Codecs.
            Default value: WAV.
        :param f: Optional. The speech audio formats.
            Allows values: see Audio Formats.
            Default value: 8khz_8bit_mono.
        :param ssml: Optional. The SSML textual content format (see SSML documentation).
            Allows values: true and false.
            Default value: false.
        :param b64: Optional. Defines output as a Base64 string format (for an internet browser playing).
            Allows values: true and false.
            Default value: false.
        :return: The result file as bytes
        """
        if isinstance(out, str):
            out = Path(out)

        if not out.parent.exists():
            logger.error(f"'{str(out.parent)}' not found!")
            return

        if out.exists():
            logger.warn(f"'{out.name}' already exists. It will be overwritten!")

        if content := self.to_bytes(src, hl, v, r, c, f, ssml=ssml, b64=b64):
            with out.open("wb") as _out:
                _out.write(content)

            return out

    def to_bytes(
        self,
        src: str,
        hl: Optional[str] = None,
        v: Optional[str] = None,
        r: Optional[int] = None,
        c: Optional[str] = None,
        f: Optional[str] = None,
        *,
        ssml: Optional[bool] = None,
        b64: Optional[bool] = None,
    ) -> bytes | None:
        """
        :param src: The textual content for converting to speech (length limited by 100KB).
        :param hl: Mandatory. The textual content language.
            Allows values: see Languages.
        :param v: Optional. The speech voice.
            Allows values: see Voices.
            Default value: depends on a language.
        :param r: Optional. The speech rate (speed).
            Allows values: from -10 (slowest speed) up to 10 (fastest speed).
            Default value: 0 (normal speed).
        :param c: Optional. The speech audio codec.
            Allows values: see Audio Codecs.
            Default value: WAV.
        :param f: Optional. The speech audio formats.
            Allows values: see Audio Formats.
            Default value: 8khz_8bit_mono.
        :param ssml: Optional. The SSML textual content format (see SSML documentation).
            Allows values: true and false.
            Default value: false.
        :param b64: Optional. Defines output as a Base64 string format (for an internet browser playing).
            Allows values: true and false.
            Default value: false.
        :return: The result file as bytes
        """

        return self._request(
            src=src,
            hl=hl or self.lang,
            v=v or self.voice,
            r=r or self.rate,
            c=c or self.codec,
            f=f or self.format,
            ssml=ssml or self.ssml,
            b64=b64 or self.b64,
        )



ar_eg = "ar-eg"
"""Arabic (Egypt)"""

ar_sa = "ar-sa"
"""Arabic (Saudi Arabia)"""

bg_bg = "bg-bg"
"""Bulgarian"""

ca_es = "ca-es"
"""Catalan"""

zh_cn = "zh-cn"
"""Chinese (China)"""

zh_hk = "zh-hk"
"""Chinese (Hong Kong)"""

zh_tw = "zh-tw"
"""Chinese (Taiwan)"""

hr_hr = "hr-hr"
"""Croatian"""

cs_cz = "cs-cz"
"""Czech"""

da_dk = "da-dk"
"""Danish"""

nl_be = "nl-be"
"""Dutch (Belgium)"""

nl_nl = "nl-nl"
"""Dutch (Netherlands)"""

en_au = "en-au"
"""English (Australia)"""

en_ca = "en-ca"
"""English (Canada)"""

en_gb = "en-gb"
"""English (Great Britain)"""

en_in = "en-in"
"""English (India)"""

en_ie = "en-ie"
"""English (Ireland)"""

en_us = "en-us"
"""English (United States)"""

fi_fi = "fi-fi"
"""Finnish"""

fr_ca = "fr-ca"
"""French (Canada)"""

fr_fr = "fr-fr"
"""French (France)"""

fr_ch = "fr-ch"
"""French (Switzerland)"""

de_at = "de-at"
"""German (Austria)"""

de_de = "de-de"
"""German (Germany)"""

de_ch = "de-ch"
"""German (Switzerland)"""

el_gr = "el-gr"
"""Greek"""

he_il = "he-il"
"""Hebrew"""

hi_in = "hi-in"
"""Hindi"""

hu_hu = "hu-hu"
"""Hungarian"""

id_id = "id-id"
"""Indonesian"""

it_it = "it-it"
"""Italian"""

ja_jp = "ja-jp"
"""Japanese"""

ko_kr = "ko-kr"
"""Korean"""

ms_my = "ms-my"
"""Malay"""

nb_no = "nb-no"
"""Norwegian"""

pl_pl = "pl-pl"
"""Polish"""

pt_br = "pt-br"
"""Portuguese (Brazil)"""

pt_pt = "pt-pt"
"""Portuguese (Portugal)"""

ro_ro = "ro-ro"
"""Romanian"""

ru_ru = "ru-ru"
"""Russian"""

sk_sk = "sk-sk"
"""Slovak"""

sl_si = "sl-si"
"""Slovenian"""

es_mx = "es-mx"
"""Spanish (Mexico)"""

es_es = "es-es"
"""Spanish (Spain)"""

sv_se = "sv-se"
"""Swedish"""

ta_in = "ta-in"
"""Tamil"""

th_th = "th-th"
"""Thai"""

tr_tr = "tr-tr"
"""Turkish"""

vi_vn = "vi-vn"
"""Vietnamese"""



import logging as _logging

logger = _logging.getLogger(__name__)
__stream_handler = _logging.StreamHandler()
__stream_handler.setFormatter(_logging.Formatter("%(levelname)s - %(message)s"))
logger.addHandler(__stream_handler)

from .voicerss import VoiceRSS
